/* --COPYRIGHT--,BSD
// BSD-3-Clause
//
// Copyright (c) 2018, Texas Instruments Incorporated
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
//
// * Redistributions of source code must retain the above copyright
// notice, this list of conditions and the following disclaimer.
//
// * Redistributions in binary form must reproduce the above copyright
// notice, this list of conditions and the following disclaimer in the
// documentation and/or other materials provided with the distribution.
//
// * Neither the name of Texas Instruments Incorporated nor the names of
// its contributors may be used to endorse or promote products derived
// from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
// PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
// OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
// WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
// OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
// EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/
//*****************************************************************************
//! \file   WakeOnProximity_Sensor.c
//
// This is a 'custom' sensor configuration that enables all electrodes on the
// CAPT-BSWP panel to simultaneously act as proximity sensors to increase
// the boards sensitivity while in Wake On Proximity mode.
// The reason is the dedicated prox sensor works well for normal HMI button
// interface, which is typically a slower button press.  For the media player,
// the user can interact the the buttons, slider or wheel very rapidly and miss
// a gesture due to the dedicated proximity sensor's less sensitive response.
//
// This sensor structure is the base sensor, consisting of the first four
// buttons or elements, one from each measurement block.  Reason is the FSM
// (finite state machine) is limited to scanning a single cycle.  In this case,
// the first four buttons.
//
// Enabling the other electrodes as self capacitive electrodes before entering
// Wake On Proximity mode allows all electrodes to be active during the single
// measurement cycle, thus creating a larger 'virtual' proximity sensor.
//
//  Group:          MSP
//  Target Device:  MSP430FR2633
//
//  (C) Copyright 2018, Texas Instruments, Inc.
//#############################################################################
// TI Release: 1.00.00.00
// Release Date: Dec 04, 2018
//#############################################################################
//
//*****************************************************************************

#include "CAPT_UserConfig.h"

//*****************************************************************************
//
//! Captivate Element Definitions
//! All elements in this application are defined below.
//! Each element has 3 components:
//!  1) a raw count array (One index per freq. scanned) (uint16_t)
//!  2) a tuning array (One index per freq. scanned) (tCaptivateElementTuning)
//!  3) a element structure (tElement)
//
//*****************************************************************************
// Sensor: keypadSensor, Element: E00
uint16_t WakeOnProximitySensor_E00_RawCnts[CAPT_SELF_FREQ_CNT];
tCaptivateElementTuning WakeOnProximitySensor_E00_Tuning[CAPT_SELF_FREQ_CNT];
tElement WakeOnProximitySensor_E00 =
{
    .ui8RxPin = 0,
    .ui8RxBlock = 0,
    .ui8TouchThreshold = 10,
    .pRawCount = WakeOnProximitySensor_E00_RawCnts,
    .pTuning = WakeOnProximitySensor_E00_Tuning,
};

// Sensor: keypadSensor, Element: E01
uint16_t WakeOnProximitySensor_E01_RawCnts[CAPT_SELF_FREQ_CNT];
tCaptivateElementTuning WakeOnProximitySensor_E01_Tuning[CAPT_SELF_FREQ_CNT];
tElement WakeOnProximitySensor_E01 =
{
    .ui8RxPin = 0,
    .ui8RxBlock = 1,
    .ui8TouchThreshold = 10,
    .pRawCount = WakeOnProximitySensor_E01_RawCnts,
    .pTuning = WakeOnProximitySensor_E01_Tuning,
};

// Sensor: keypadSensor, Element: E02
uint16_t WakeOnProximitySensor_E02_RawCnts[CAPT_SELF_FREQ_CNT];
tCaptivateElementTuning WakeOnProximitySensor_E02_Tuning[CAPT_SELF_FREQ_CNT];
tElement WakeOnProximitySensor_E02 =
{
    .ui8RxPin = 0,
    .ui8RxBlock = 2,
    .ui8TouchThreshold = 10,
    .pRawCount = WakeOnProximitySensor_E02_RawCnts,
    .pTuning = WakeOnProximitySensor_E02_Tuning,
};

// Sensor: keypadSensor, Element: E03
uint16_t WakeOnProximitySensor_E03_RawCnts[CAPT_SELF_FREQ_CNT];
tCaptivateElementTuning WakeOnProximitySensor_E03_Tuning[CAPT_SELF_FREQ_CNT];
tElement WakeOnProximitySensor_E03 =
{
    .ui8RxPin = 0,
    .ui8RxBlock = 3,
    .ui8TouchThreshold = 10,
    .pRawCount = WakeOnProximitySensor_E03_RawCnts,
    .pTuning = WakeOnProximitySensor_E03_Tuning,
};

//*****************************************************************************
//
//! Captivate Time Cycle Definitions
//! All time cycles in this application are defined below.  Time cycles are
//! groups of elements that are measured together in parallel in one time slot.
//! Each cycle has 2 components:
//!  1) an element pointer array to the member elements (tElement*)
//!  2) a cycle structure (tCycle)
//
//*****************************************************************************
// Time Cycle: WakeOnProximitySensor_C00
tElement* WakeOnProximitySensor_C00_Elements[4] =
{
    &WakeOnProximitySensor_E00,
    &WakeOnProximitySensor_E01,
    &WakeOnProximitySensor_E02,
    &WakeOnProximitySensor_E03,
};
tCycle WakeOnProximitySensor_C00 =
{
    .ui8NrOfElements = 4,
    .pElements = WakeOnProximitySensor_C00_Elements,
};

//*****************************************************************************
//
//! Captivate Sensor Definitions
//! All sensors in this application are defined below.  Sensors are
//! groups of time cycles that utilize raw measurement data to create an
//! abstract sensor type, such as a button, slider, wheel, or prox sensor.
//! Each sensor has 3 components:
//!  1) a cycle pointer array to the member time cycles (tCycle*)
//!  2) a sensor-specific parameter structure (tGenericSensorParams)
//!  3) a sensor structure (tSensor)
//
//*****************************************************************************
//Sensor: keypadSensor
const tCycle* WakeOnProximitySensor_Cycles[1] =
{
    &WakeOnProximitySensor_C00,
};

tButtonSensorParams WakeOnProximitySensor_Params;
tSensor WakeOnProximitySensor =
{
    // Basic Properties
    .TypeOfSensor = eButtonGroup,
    .SensingMethod = eSelf,
    .DirectionOfInterest = eDOIDown,
    .pvCallback = NULL,
    .ui8NrOfCycles = 1,
    .pCycle = WakeOnProximitySensor_Cycles,
    .pSensorParams = (tGenericSensorParams*)&WakeOnProximitySensor_Params,
    // Conversion Control Parameters
    .ui16ConversionCount = 350,
    .ui16ConversionGain = 100,
    .ui8FreqDiv = 2,
    .ui8ChargeLength = 0,
    .ui8TransferLength = 0,
    .bModEnable = false,
    .ui8BiasControl = 3,
    .bCsDischarge = true,
    .bLpmControl = false,
    .ui8InputSyncControl = 0,
    .bTimerSyncControl = false,
    .bIdleState = true,
    // Tuning  Parameters
    .ui16ProxThreshold = 10,
    .ui16NegativeTouchThreshold = 20,
    .ui16ErrorThreshold = 1000,
    .ui16TimeoutThreshold = 1980,
    .ProxDbThreshold.DbIn = 1,
    .ProxDbThreshold.DbOut = 1,
    .TouchDbThreshold.DbIn = 0,
    .TouchDbThreshold.DbOut = 0,
    .bCountFilterEnable = true,
    .ui8CntBeta = 1,
    .bSensorHalt = false,
    .bPTSensorHalt = true,
    .bPTElementHalt = true,
    .ui8LTABeta = 7,
    .bReCalibrateEnable = true,
};

/*
 * Proximity channel control and wrapper functions
 *
 */
// ============================================================================

// Note: The WakeOnProximitySensor uses {block,pin} -> {0,0},{1,0},{2,0},{3,0}
// by default.  All additional pins on the FR2633 are temporarily configured
// as RX channels.
#define WOP_RX_IO_COUNT     (12)
uint8_t WOP_RX_IO_Table[][2] =
{
 {0,1},
 {0,2},
 {0,3},
 {1,1},
 {1,2},
 {1,3},
 {2,1},
 {2,2},
 {2,3},
 {3,1},
 {3,2},
 {3,3},
};


/*
 * SELF CAP SENSOR ONLY
 * Set active the selected RX block,pin combination for the FSM to include it in a conversion measurement.
 */
void activate_WOP_Sensor_RX_Channels(void)
{
    uint8_t ui8Index;

    for(ui8Index = 0; ui8Index < WOP_RX_IO_COUNT; ui8Index++)
    {
        // Setting the channel active = 1 enables the corresponding CAPT IO to be included in the measurement
        CAPT_setChannelActive(WOP_RX_IO_Table[ui8Index][0], WOP_RX_IO_Table[ui8Index][1]);
    }

}

/*
 * SELF CAP SENSOR ONLY
 * Set inactive the selected RX block,pin combination
 */
void deactivate_WOP_Sensor_RX_Channels(void)
{
    uint8_t ui8Index;

    for(ui8Index = 0; ui8Index < WOP_RX_IO_COUNT; ui8Index++)
    {
        // Clearing the channel active = 0 disables the corresponding CAPT IO and will not be included in the measurement
        CAPT_clearChannelActive(WOP_RX_IO_Table[ui8Index][0], WOP_RX_IO_Table[ui8Index][1]);
    }

}

/*
 * SELF CAP SENSOR ONLY
 * Set active the selected RX block,pin combination for the FSM to include it in a conversion measurement.
 */
void enable_WOP_Sensor_RX_Channels(void)
{
    uint8_t ui8Index;

    for(ui8Index = 0; ui8Index < WOP_RX_IO_COUNT; ui8Index++)
    {
        // Setting the channel active = 1 enables the corresponding CAPT IO to be included in the measurement
        CAPT_setChannelEnable(WOP_RX_IO_Table[ui8Index][0], WOP_RX_IO_Table[ui8Index][1]);
    }

}

/*
 * SELF CAP SENSOR ONLY
 * Set inactive the selected RX block,pin combination
 */
void disable_WOP_Sensor_RX_Channels(void)
{
    uint8_t ui8Index;

    for(ui8Index = 0; ui8Index < WOP_RX_IO_COUNT; ui8Index++)
    {
        // Clearing the channel active = 0 disables the corresponding CAPT IO and will not be included in the measurement
        CAPT_clearChannelEnable(WOP_RX_IO_Table[ui8Index][0], WOP_RX_IO_Table[ui8Index][1]);
    }

}

/*
 * SELF CAP SENSOR ONLY
 * Set active the selected RX block,pin combination for the FSM to include it in a conversion measurement.
 */
void init_WOP_Sensor_RX_Channels(void)
{
    uint8_t ui8Index;

    for(ui8Index = 0; ui8Index < WOP_RX_IO_COUNT; ui8Index++)
    {
        // Setting the channel on state = 1 sets the corresponding CAPT IO = RX mode
        CAPT_setChannelOnState(WOP_RX_IO_Table[ui8Index][0], WOP_RX_IO_Table[ui8Index][1]);
    }

}

/*
 * WOP_Callback
 * Use this callback function during the tuning process to send
 * sensor data to the captivate design center.
 * Not used during normal operation.
 * Automatically enabled when CAPT_WAKEONPROX_ENABLE  (false) in CAPT_UserConfig.h.
 *
 * User Data array defined
 *
 * userData[0] = E00 count
 * userData[1] = E01 count
 * userData[2] = E02 count
 * userData[3] = E03 count
 * userData[4] = E00 LTA
 * userData[5] = E00 LTA
 * userData[6] = E00 LTA
 * userData[7] = E00 LTA
 * userData[8] = E00 delta count
 * userData[9] = E01 delta count
 * userData[10] = E02 delta count
 * userData[11] = E03 delta count
 * userData[12] = sensor proximity threshold
 * userData[13] = count average
 * userData[14] = LTA average
 * userData[15] = delta count average
 *
 */
void WOP_Sensor_Callback(tSensor* pSensor)
{

    static uint16_t userData[16];
    uint8_t ui8Index;
    uint16_t ui16Count_Average;
    uint16_t ui16LTA_Average;
    uint16_t ui16Delta;

    tElement* pElement = pSensor->pCycle[0]->pElements[0];

    // If you want to see the actual count, add it here
    // 0..3
    ui16Count_Average = 0;
    for(ui8Index=0; ui8Index < 4; ui8Index++)
    {
        userData[ui8Index] = pElement[ui8Index].filterCount.ui16Natural;
        ui16Count_Average += pElement[ui8Index].filterCount.ui16Natural;

    }
    ui16Count_Average >>= 2;

    // If you want to see the actual LTA, add it here
    // 4..7
    ui16LTA_Average = 0;
    for(ui8Index=0; ui8Index < 4; ui8Index++)
    {
        userData[ui8Index + 4] = pElement[ui8Index].LTA.ui16Natural;
        ui16LTA_Average += pElement[ui8Index].LTA.ui16Natural;
    }
    ui16LTA_Average >>= 2;


    // Calculate the delta counts
    // 8..11
    ui16Delta = 0;
    for(ui8Index=0; ui8Index < 4; ui8Index++)
    {
        if(pElement[ui8Index].filterCount.ui16Natural > pElement[ui8Index].LTA.ui16Natural)
            userData[ui8Index + 8] = 0;
        else
            userData[ui8Index + 8] = pElement[ui8Index].LTA.ui16Natural - pElement[ui8Index].filterCount.ui16Natural;

        ui16Delta += userData[ui8Index + 8];

    }
    ui16Delta >>= 2;


    // Add the proximity threshold
    // 12
    userData[12] = ui16LTA_Average - pSensor->ui16ProxThreshold;

    // Add the count average
    // 13
    userData[13] = ui16Count_Average;

    // Add the LTA average
    // 14
    userData[14] = ui16LTA_Average;

    // Add the delta count average
    // 15
    userData[15] = ui16Delta;

    CAPT_writeGeneralPurposeData(userData, 16);
}

