/* --COPYRIGHT--,BSD
// BSD-3-Clause
//
// Copyright (c) 2018, Texas Instruments Incorporated
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
//
// * Redistributions of source code must retain the above copyright
// notice, this list of conditions and the following disclaimer.
//
// * Redistributions in binary form must reproduce the above copyright
// notice, this list of conditions and the following disclaimer in the
// documentation and/or other materials provided with the distribution.
//
// * Neither the name of Texas Instruments Incorporated nor the names of
// its contributors may be used to endorse or promote products derived
// from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
// PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
// OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
// WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
// OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
// EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/
//#############################################################################
//
//! \file   media_player.c
//
//! \brief  Media player demo application code
//
//  Group:          MSP
//  Target Device:  MSP430FR2633
//
//  (C) Copyright 2018, Texas Instruments, Inc.
//#############################################################################
// TI Release: 1.00.00.00
// Release Date: Dec 04, 2018
//#############################################################################

//*****************************************************************************
// Includes
//*****************************************************************************
#include <string.h>
#include <stdio.h>
#include <CAPT_bsp.h>
#include <I2CSlave.h>
#include <UART.h>
#include <gestures.h>
#include <media_player.h>
#include <WakeOnProximity_Sensor.h> // do not delete this!


//*****************************************************************************
// Definitions
//*****************************************************************************
// The following defines are derived from the USB.ORG USB HID Usage Table
// Keyboard/Keypad Page (0x07)
// https://www.usb.org/sites/default/files/documents/hut1_12v2.pdf

// Key values described in the document table are 8-bit values (0x00 - 0xFF).
// In order to represent both the key value and it's modifier, a 16-bit value
// is used to represent both. The lower 8-bits represent the key and the upper
// 8-bits represent the modifier. In this demo, only use the LEFT-CTRL is used.

#define KEY_B                       0x0005
#define KEY_F                       0x0009
#define KEY_P                       0x0013
#define KEY_S                       0x0016
#define KEY_F7                      0x0040
#define KEY_F8                      0x0041
#define KEY_F9                      0x0042
#define KEY_MOD_LCTRL               0xE000

// These defines map the key values to their corresponding media functions.
#define KEY_MEDIA_STOP              KEY_MOD_LCTRL | KEY_S
#define KEY_MEDIA_PLAYPAUSE         KEY_MOD_LCTRL | KEY_P
#define KEY_MEDIA_PREVIOUS_TRACK    KEY_MOD_LCTRL | KEY_B
#define KEY_MEDIA_NEXT_TRACK        KEY_MOD_LCTRL | KEY_F
#define KEY_MEDIA_MUTE              KEY_F7
#define KEY_MEDIA_VOLUME_UP         KEY_F9
#define KEY_MEDIA_VOLUME_DOWN       KEY_F8
#define KEY_MEDIA_RESERVED          0x0000 //(future use)

//*****************************************************************************
// Macros
//*****************************************************************************
#define LO_BYTE(x)      (x & 0xFF)
#define HI_BYTE(x)      ((x & 0xFF00) >> 8)

//*****************************************************************************
// Global Variables
//*****************************************************************************

//*****************************************************************************
// ! Function Declarations
// ! To support the media demo, all three sensors (slider, wheel and buttons)
// ! are compiled by default.  To not include any of the sensors, see the file
// ! gesture_definitions.h for information.
//*****************************************************************************
#if USE_BUTTONGROUP==true
static void keypadSensorHandler(tSensor *pSensor);
extern tSensorGesture  ButtonGroupGesture;
#endif
#if USE_SLIDER==true
static void sliderSensorHandler(tSensor *pSensor);
extern tSensorGesture  SliderGesture;
#endif
#if USE_WHEEL==true
static void wheelSensorHandler(tSensor *pSensor);
extern tSensorGesture  WheelGesture;
#endif

// WOP sensor data and Gesture data cannot be used at same time to send data
// to the GUI. Produce warning message if both enabled.
// See WakeOnProximity_Sensor.h and media_player.h for definitions.
#if((SEND_WOP_SENSOR_DATA==true) && (CAPT_WAKEONPROX_ENABLE==false) && (SEND_GESTURE_DATA==true))
#error("Cannot send Wake On Proximity and Gesture data at same time.  Disable one or the other and re-compile! ")
#endif

void sendGestureStatus_GUI(tSensor* pSensor, tSensorGesture *pGesture);

//*****************************************************************************
// I2C Interface
//*****************************************************************************
// This demo sends two 16-bit key values (current and previous) = 4 bytes
uint8_t     TXBuffer[4];
uint8_t     RXBuffer[4];

/*
 * CREATE A LOCAL INSTANCE OF I2C SLAVE PARAMETERS
 */
static const tI2CSlavePort slaveI2c =
{
    .pbReceiveCallback = NULL,
    .pvErrorCallback = NULL,
    .pReceiveBuffer = RXBuffer,
    .ui16ReceiveBufferSize = sizeof(RXBuffer),
    .bSendReadLengthFirst = false
};


//*****************************************************************************
//
// ! Media Key Lookup Table
// ! Each gesture type is mapped to a specific media player key
// ! Gesture used as Index for this table
// ! Example: TAP gesture = 0 ---> keyGestureTable[0] = KEY_MEDIA_PLAYPAUSE
//
//*****************************************************************************
/*
 */
uint16_t keyGestureTable[] =
{
     KEY_MEDIA_PLAYPAUSE,       //Corresponding gesture = TAP
     KEY_MEDIA_MUTE,            //Corresponding gesture = DTAP
     KEY_MEDIA_RESERVED,        //Corresponding gesture = TAP-N-HOLD (if used)
     KEY_MEDIA_PREVIOUS_TRACK,  //Corresponding gesture = SWIPE LEFT
     KEY_MEDIA_NEXT_TRACK,      //Corresponding gesture = SWIPE RIGHT
     KEY_MEDIA_VOLUME_DOWN,     //Corresponding gesture = SLIDE LEFT
     KEY_MEDIA_VOLUME_UP,       //Corresponding gesture = SLIDE RIGHT
};


//*****************************************************************************
//
//! Enumeration for each possible emulated key state
//
//*****************************************************************************
typedef enum{

    eEmulatedKeyNotPressed,
    eEmulatedKeyPressed,
    eEmulatedKeyReleased,

}emulatedKeyStates;

//*****************************************************************************
// Function Implementations
//*****************************************************************************

void Demo_Init(void)
{
#if USE_BUTTONGROUP==true
    MAP_CAPT_registerCallback(&keypadSensor, &keypadSensorHandler);
#endif
#if USE_SLIDER==true
    MAP_CAPT_registerCallback(&sliderSensor, &sliderSensorHandler);
#endif
#if USE_WHEEL==true
    MAP_CAPT_registerCallback(&wheelSensor, &wheelSensorHandler);
#endif

// I2C is reserved for the HOST MCU and cannot be used with the CapTIvate Design Center.
// Select UART instead for gesture tuning
#if (CAPT_INTERFACE==__CAPT_BULKI2C_INTERFACE__) // CAPT_UserConfig.h
#error "I2C interface reserved for HOST MCU: USE CAPT_UART_INTERFACE TO COMMUNICATE WITH DESIGN CENTER OR CAPT_NONE_INTERFACE"
#else

    I2CSlave_openPort(&slaveI2c);

    // Assign the transmit buffer
    I2CSlave_setTransmitBuffer(TXBuffer, sizeof(TXBuffer));

#endif
}


#if USE_SLIDER==true
//*****************************************************************************
//
// ! Slider handler
// ! This Handler is the callback function for this sensor and is called
// ! automatically each time after the sensor is scanned.
// ! Handler must first be registered as the sensor's callback.
// ! See Demo_init() to see example usage.
//
//*****************************************************************************
void sliderSensorHandler(tSensor *pSensor)
{
    static uint16_t current_key_status;
    static uint16_t previous_key_status;
    static uint8_t emulateKeyState = eEmulatedKeyNotPressed;
    gesture_t gesture;

    if (FIRST_TOUCH)
    {
        LED1_ON;

    }
    else if(EXIT_TOUCH)
    {
        LED1_OFF;

    }

    gesture = processSliderGesture(pSensor);

#if SEND_GESTURE_DATA==true     // Refer to gesture_definitions.h

#if (CAPT_INTERFACE==__CAPT_BULKI2C_INTERFACE__)
#error "I2C interface reserved for HOST MCU: USE CAPT_UART_INTERFACE TO COMMUNICATE WITH DESIGN CENTER OR CAPT_NONE_INTERFACE"
#elif ((CAPT_INTERFACE==__CAPT_NONE_INTERFACE__))
#warning "Select __CAPT_UART_INTERFACE__ in CAPT_UserConfig.h to enable communications with CapTIvate Design Center GUI"
#elif (CAPT_INTERFACE==__CAPT_UART_INTERFACE__)

    // Use UART_A0 to output gesture tuning data in the GUI
    // Compile time option - see media_player.h
    sendGestureStatus_GUI(pSensor, &SliderGesture);
#endif
#endif

    switch(emulateKeyState)
    {
    case eEmulatedKeyNotPressed:

        // Check if valid gesture
        if(gesture == eNoGesture)
        {
            LED2_OFF;
            return;
        }

        LED2_ON;

        current_key_status = 0x0000;
        current_key_status = keyGestureTable[gesture];

        // clear the status bits to represent previous state
        previous_key_status = 0x0000;


        // Populate the buffer
        TXBuffer[0] = LO_BYTE(current_key_status);
        TXBuffer[1] = HI_BYTE(current_key_status);
        TXBuffer[2] = LO_BYTE(previous_key_status);
        TXBuffer[3] = HI_BYTE(previous_key_status);

        // Signal the master to come read the data
        I2CSlave_setRequestFlag();

        // The next time sensor is scanned we want to be in the next key state
        emulateKeyState = eEmulatedKeyPressed;

        break;

    case eEmulatedKeyPressed: // Emulate Key Press
        // In previous state, we emulated a key press and reported the Gesture.
        // To prevent a "repeated key press", we report Key Released on the following sample.

        previous_key_status = current_key_status;
        current_key_status = 0x0000;

        TXBuffer[0] = LO_BYTE(current_key_status);
        TXBuffer[1] = HI_BYTE(current_key_status);
        TXBuffer[2] = LO_BYTE(previous_key_status);
        TXBuffer[3] = HI_BYTE(previous_key_status);

        I2CSlave_setRequestFlag();

        emulateKeyState = eEmulatedKeyNotPressed;
        break;

    default:  break;

    }// end switch/case
}
#endif


#if USE_WHEEL==true
//*****************************************************************************
//
// ! Wheel handler
// ! This Handler is the callback function for this sensor and is called
// ! automatically each time after the sensor is scanned.
// ! Handler must first be registered as the sensor's callback.
// ! See Demo_init() to see example usage.
//
//*****************************************************************************
void wheelSensorHandler(tSensor *pSensor)
{
    static uint16_t current_key_status;
    static uint16_t previous_key_status;
    static uint8_t emulateKeyState = eEmulatedKeyNotPressed;
    gesture_t gesture;

    if (FIRST_TOUCH)
    {
        LED1_ON;

    }
    else if(EXIT_TOUCH)
    {
        LED1_OFF;

    }

    gesture = processWheelGesture(pSensor);

#if SEND_GESTURE_DATA==true     // Refer to gesture_definitions.h

#if (CAPT_INTERFACE==__CAPT_BULKI2C_INTERFACE__)
#error "I2C interface reserved for HOST MCU: USE CAPT_UART_INTERFACE TO COMMUNICATE WITH DESIGN CENTER OR CAPT_NONE_INTERFACE"
#elif ((CAPT_INTERFACE==__CAPT_NONE_INTERFACE__))
#warning "Select __CAPT_UART_INTERFACE__ in CAPT_UserConfig.h to enable communications with CapTIvate Design Center GUI"
#elif (CAPT_INTERFACE==__CAPT_UART_INTERFACE__)

    // Use UART_A0 to output gesture tuning data in the GUI
    // Compile time option - see media_player.h
    sendGestureStatus_GUI(pSensor, &WheelGesture);
#endif
#endif

    switch(emulateKeyState)
    {
    case eEmulatedKeyNotPressed:

        // Check if valid gesture
        if(gesture == eNoGesture)
        {
            LED2_OFF;
            return;
        }

        LED2_ON;

        current_key_status = 0x0000;
        current_key_status = keyGestureTable[gesture];

        // clear the status bits to represent previous state
        previous_key_status = 0x0000;

        // Populate the buffer
        TXBuffer[0] = LO_BYTE(current_key_status);
        TXBuffer[1] = HI_BYTE(current_key_status);
        TXBuffer[2] = LO_BYTE(previous_key_status);
        TXBuffer[3] = HI_BYTE(previous_key_status);

        // Signal the master to come read the data
        I2CSlave_setRequestFlag();

        // The next time sensor is scanned we want to be in the next key state
        emulateKeyState = eEmulatedKeyPressed;

        break;

    case eEmulatedKeyPressed: // Emulate Key Press

        // In previous state, we emulated a key press and reported the Gesture.
        // To prevent a "repeated key press", we report Key Released on the following sample.
        previous_key_status = current_key_status;
        current_key_status = 0x0000;

        TXBuffer[0] = LO_BYTE(current_key_status);
        TXBuffer[1] = HI_BYTE(current_key_status);
        TXBuffer[2] = LO_BYTE(previous_key_status);
        TXBuffer[3] = HI_BYTE(previous_key_status);

        I2CSlave_setRequestFlag();

        emulateKeyState = eEmulatedKeyNotPressed;
        break;

    default:  break;

    }// end switch/case

}
#endif

#if USE_BUTTONGROUP==true
//*****************************************************************************
//
// ! Button handler
// ! This Handler is the callback function for this sensor and is called
// ! automatically each time after the sensor is scanned.
// ! Handler must first be registered as the sensor's callback.
// ! See Demo_init() to see example usage.
//
//*****************************************************************************
void keypadSensorHandler(tSensor *pSensor)
{
    static uint16_t current_key_status;
     static uint16_t previous_key_status;
     static uint8_t emulateKeyState = eEmulatedKeyNotPressed;
     gesture_t gesture;

     if (FIRST_TOUCH)
     {
         LED1_ON;

     }
     else if(EXIT_TOUCH)
     {
         LED1_OFF;

     }

     gesture = processButtonGroupGesture(pSensor);

#if SEND_GESTURE_DATA==true     // Refer to gesture_definitions.h

#if (CAPT_INTERFACE==__CAPT_BULKI2C_INTERFACE__)
#error "I2C interface reserved for HOST MCU: USE CAPT_UART_INTERFACE TO COMMUNICATE WITH DESIGN CENTER OR CAPT_NONE_INTERFACE"
#elif ((CAPT_INTERFACE==__CAPT_NONE_INTERFACE__))
#error "Select __CAPT_UART_INTERFACE__ in CAPT_UserConfig.h to enable communications with CapTIvate Design Center GUI"
#elif (CAPT_INTERFACE==__CAPT_UART_INTERFACE__)

     // Use UART_A0 to output gesture tuning data in the GUI
     // Compile time option - see media_player.h
     sendGestureStatus_GUI(pSensor, &ButtonGroupGesture);
#endif
#endif

     switch(emulateKeyState)
     {
     case eEmulatedKeyNotPressed:

          // Check if valid gesture
         if(gesture == eNoGesture)
         {
             LED2_OFF;
             return;
         }

         LED2_ON;

         current_key_status = 0x0000;
         current_key_status = keyGestureTable[gesture];

         // clear the status bits to represent previous state
         previous_key_status = 0x0000;

         // Populate the buffer
         TXBuffer[0] = LO_BYTE(current_key_status);
         TXBuffer[1] = HI_BYTE(current_key_status);
         TXBuffer[2] = LO_BYTE(previous_key_status);
         TXBuffer[3] = HI_BYTE(previous_key_status);

         // Signal the master to come read the data
         I2CSlave_setRequestFlag();

         // The next time sensor is scanned we want to be in the next key state
         emulateKeyState = eEmulatedKeyPressed;

         break;

     case eEmulatedKeyPressed: // Emulate Key Press
         // In previous state, we emulated a key press and reported the Gesture.
         // To prevent a "repeated key press", we report Key Released on the following sample.
         previous_key_status = current_key_status;
         current_key_status = 0x0000;

         TXBuffer[0] = LO_BYTE(current_key_status);
         TXBuffer[1] = HI_BYTE(current_key_status);
         TXBuffer[2] = LO_BYTE(previous_key_status);
         TXBuffer[3] = HI_BYTE(previous_key_status);

         I2CSlave_setRequestFlag();

         emulateKeyState = eEmulatedKeyNotPressed;
         break;

     default:  break;
     }// end switch/case
}
#endif


#if SEND_GESTURE_DATA==true
/*
 * ! debugGesture
 * ! This method sends gesture data to CapTIvate Design Center where it can be plotted
 * ! in the GUI's oscilloscope view.
 * ! Use this to debug gesture state machine and tune gesture parameters.
 * ! IMPORTANT: Can only be used with one sensor at a time, so only compile the sensor
 * ! of interest when using this function.
 *
 */
#define TOUCH_AMPLITUDE         16
#define GESTURE_AMPLITUDE       32
void sendGestureStatus_GUI(tSensor* pSensor, tSensorGesture *pGesture)
{
    static uint16_t data[] = {0,0,0,0,0,0,0,0,0};

    gesture_t Gesture = pGesture->Gesture;

    // Clear previous gesture bits
    memset(data, 0, sizeof(data));

    data[0] = pSensor->bSensorTouch * TOUCH_AMPLITUDE;  // 000 on the GUI: scale the touch status x 15 over the others so we can see it on the plot
    data[1] = pGesture->ui16SampleCount;                // 001 on the GUI

    if(Gesture == eSingleTap)
    {
        data[2] = GESTURE_AMPLITUDE;                   // 002 on the GUI
        _no_operation();
    }
    else if(Gesture == eDoubleTap)
    {
        data[3] = GESTURE_AMPLITUDE;                   // 003 on the GUI
        _no_operation();
    }
    else if(Gesture == eSlideLeft)
    {
        data[4] = GESTURE_AMPLITUDE;                   // 004 on the GUI
        _no_operation();
    }
    else if(Gesture == eSlideRight)
    {
        data[5] = GESTURE_AMPLITUDE;                   // 005 on the GUI
        _no_operation();
    }
    else if(Gesture == eSwipeLeft)
    {
        data[6] = GESTURE_AMPLITUDE;                   // 006 on the GUI
        _no_operation();
    }
    else if(Gesture == eSwipeRight)
    {
        data[7] = GESTURE_AMPLITUDE;                   // 007 on the GUI
        _no_operation();
    }
    else if(Gesture == eTapHold)
    {
        data[8] = GESTURE_AMPLITUDE;                   // 008 on the GUI
        _no_operation();
    }

    // Writes data to the GUI
    CAPT_writeGeneralPurposeData(data,sizeof(data)/sizeof(uint16_t));
}
#endif
