/* --COPYRIGHT--,BSD
// BSD-3-Clause
//
// Copyright (c) 2018, Texas Instruments Incorporated
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
//
// * Redistributions of source code must retain the above copyright
// notice, this list of conditions and the following disclaimer.
//
// * Redistributions in binary form must reproduce the above copyright
// notice, this list of conditions and the following disclaimer in the
// documentation and/or other materials provided with the distribution.
//
// * Neither the name of Texas Instruments Incorporated nor the names of
// its contributors may be used to endorse or promote products derived
// from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
// PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
// OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
// WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
// OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
// EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/
//#############################################################################
//
//! \file   gesture_definitions.h
//
//! \brief  Gesture algorithms for CapTIvate Sliders, Wheels and Buttons.
//
//  Group:          MSP
//  Target Device:  MSP430FR2633
//
//  (C) Copyright 2018, Texas Instruments, Inc.
//#############################################################################
// TI Release: 1.00.00.00
// Release Date: Dec 04, 2018
//#############################################################################

#ifndef GESTURES_H_
#define GESTURES_H_

//*****************************************************************************
// Includes
//*****************************************************************************
#include <captivate.h>
#include "gesture_definitions.h"

//*****************************************************************************
// Definitions
//*****************************************************************************
#define FIRST_TOUCH             (pSensor->bSensorTouch == true) && (pSensor->bSensorPrevTouch == false)
#define EXIT_TOUCH              (pSensor->bSensorTouch == false) && (pSensor->bSensorPrevTouch == true)
#define TOUCH                   (pSensor->bSensorTouch == true)
#define NO_TOUCH                (pSensor->bSensorTouch == false) && (pSensor->bSensorPrevTouch == false)

//*****************************************************************************
//
//! An enumeration for each of the state-machine processing states
//
//*****************************************************************************
typedef enum {
    eStateBackGround,
    eStateFingerTap,
    eStateFingerPress,
    eStateFingerSlide,
    eStateFingerHold,
    eStateWaitFingerRelease,
} eGestureStates;
typedef uint8_t gestureState_t;

//*****************************************************************************
//
//! Enumeration for each of the supported Gestures
//
//*****************************************************************************
typedef enum{
    eSingleTap = 0,
    eDoubleTap,
    eTapHold,
    eSwipeLeft,
    eSwipeRight,
    eSlideLeft,
    eSlideRight,
    eNoGesture = 0xFF
} eGestures;
typedef uint8_t gesture_t;


//*****************************************************************************
//
//! Gesture Parameter Structure
//! Controls a sensor's gesture timing and thresholds
//! Each sensor uses a copy of this structure for its own specific gesture
//
//*****************************************************************************
typedef struct
{

//*****************************************************************************
//! TOUCH SAMPLE MINIMUM COUNT (Slider, wheel and button)
//!
//! Minimum (touch == true) samples required for a valid TAP, SWIPE or SLIDE
//! If #samples => MINIMUM then valid
//! If #samples < MINIMUM then not valid (finger touch not long enough)
//*****************************************************************************
uint16_t ui16TouchSampleCount_Min;

//*****************************************************************************
//! TOUCH SAMPLE MAXIMUM COUNT (Slider, wheel and button)
//!
//! Maximum (touch == true) samples allowed for valid TAP, SWIPE or SLIDE
//! If #samples => MAXIMUM then not valid (finger touch too long)
//! If #samples < MAXIMUM then valid
//*****************************************************************************
uint16_t ui16TouchSampleCount_Max;

//*****************************************************************************
//! TOUCH HOLD SAMPLE MINIMUM COUNT (Button)
//!
//! Minimum (touch == true) samples required for a valid TOUCH HOLD
//! If #samples => MINIMUM then valid
//! If #samples < MINIMUM then not a TAP-HOLD
//! Note-TOUCH HOLD parameter should be set larger than TOUCH MAX
//*****************************************************************************
uint16_t ui16TouchHoldSampleCount_Min;

//*****************************************************************************
//! TOUCH SAMPLE MAXIMUM DELAY FOR DOUBLE TAP
//!
//! Def: #of samples (time) between end of first tap and start of second tap
//! Maximum samples allowed for valid DOUBLE TAP
//! If #samples < MAX then valid
//! If #samples => MAX then not valid (second finger touch came too late)
//*****************************************************************************
uint16_t ui16DoubleTapDelayCount_Max;

//*****************************************************************************
//! SWIPE SAMPLE MAXIMUM COUNT (Slider, wheel and button)
//!
//! Def: #of samples(time) between finger first touch and release
//! Note: for a valid swipe, finger must also move minimum distance (see next)
//! Maximum samples required for valid SWIPE
//! If #samples < MAX then valid
//! If #samples => MAX then not valid (finger touching too long)
//*****************************************************************************
uint16_t ui16SwipeSampleCount_Max;

//*****************************************************************************
//! SWIPE MINIMUM DISTANCE (Slider and Wheel) (Swipe)
//!
//! Def: Distance between finger's touch and release positions
//! Note: for a valid swipe, finger must also not touch too long (see prev)
//! A Minimum distance for a valid swipe
//! If delta < MIN then not valid (finger didn't move far enough)
//! If delta > MIN then valid
//*****************************************************************************
uint16_t ui16SwipeDistance_Min;

//*****************************************************************************
//! SLIDE SAMPLE MINIMUM COUNT  (Slider and Wheel) (Slide)
//!
//! A Minimum (touch == true) samples required for valid SLIDE
//! Note: Typically set > SWIPE COUNT MAX
//! If #samples < MIN then not valid (finger not touch long enough)
//! If #samples => MIN then valid
//*****************************************************************************
uint16_t ui16SlideCount_Min;

//*****************************************************************************
//! SLIDE STEP SIZE MINIMUM DISTANCE  (Slider and Wheel) (Slide)
//!
//! Minimum distance finger must move before gesture is reported
//! Note: Typically used to repeat the gesture by moving finger
//! If delta < MIN then not valid (finger didn't move far enough)
//! If delta => MIN then valid
//*****************************************************************************
uint16_t ui16SlideStepSize_Min;

//*****************************************************************************
//! FINGER MINIMUM DISTANCE  (Slider and Wheel) (Swipe and Slide)
//!
//! Minimum distance finger required for motion gesture (swipe and slide)
//! Note:  This is used to determine a motion gesture vs. TAP, DOUBLE TAP
//! If #samples < MAX then valid double tap
//! If #samples => MAX then not valid double tap
//*****************************************************************************
uint16_t ui16FingerDistance_Min;

}tSensorGestureParams;


//*****************************************************************************
//
//! Gesture structure
//! Represents a gesture's runtime parameters
//! Each sensor uses a copy of this structure for its own specific gesture
//
//*****************************************************************************
typedef struct{
    //*****************************************************************************
    //! Gesture Parameter Pointer (Slider, wheel and button)
    //!
    //! Pointer to the gesture parameters structure
    //*****************************************************************************
    const tSensorGestureParams *params;

    //*****************************************************************************
    //! Sample Count (Slider, wheel and button)
    //!
    //! Counter is updated after a sensor is updated.  This counter provides the
    //! timing required to determine each of the supported gestures.  The frequency
    //! of sample count updates is the same as sensor update rate.  Changing the
    //! sensor update rate directly impacts the sample count rate, which affects
    //! the ability to correctly determine each gesture.
    //*****************************************************************************
    uint16_t ui16SampleCount;

    //*****************************************************************************
    //! First Finger Position (Slider and wheel)
    //!
    //! On slider and wheel sensors, the position of the finger is used to help
    //! determine swipe and slide gestures.
    //! On the very first touch event, the initial position is recorded.
    //*****************************************************************************
    uint16_t ui16FingerFirstPos;

    //*****************************************************************************
    //! Previous Finger Position (Slider and wheel)
    //!
    //! On slider and wheel sensors, as the finger moves along the sensor, the
    //! last finger position is recorded as the previous and the new finger position
    //! becomes the last finger position.
    //*****************************************************************************
    uint16_t ui16FingerPreviousPos;

    //*****************************************************************************
    //! Last Finger Position (Slider and wheel)
    //!
    //! On slider and wheel sensors, this is the most recent recorded finger
    //! position. Using this position along with the First or Previous positions
    //! makes it possible to determine the total distance traveled.
    //*****************************************************************************
    uint16_t ui16FingerLastPos;

    //*****************************************************************************
    //! Delta Finger Position (Slider and wheel)
    //!
    //! On slider and wheel sensors, this is the calculated absolute difference
    //! between the Last finger position and First or Previous position.
    //*****************************************************************************
    uint16_t ui16FingerDeltaPos;

    //*****************************************************************************
    //! Gesture (Slider, wheel and button)
    //!
    //! This is the reported gesture.  Refer to the eGestures enumerations above.
    //*****************************************************************************
    gesture_t Gesture;

    //*****************************************************************************
    //! Finger Swipe Direction (Slider and wheel)
    //!
    //! On slider and wheel sensors, this is the direction the finger moved.
    //*****************************************************************************
    gesture_t SwipeDirection;

    //*****************************************************************************
    //! State Machine State (Slider, wheel and button)
    //!
    //! Keeps track of a gesture's processing state.
    //! Refer to the eGestureStates enumerations above.
    //*****************************************************************************
    gestureState_t TouchState;

    //*****************************************************************************
    //! Finger Touch Status (Slider, wheel and button)
    //!
    //! On all sensors, this is used in the gesture processing to know if the
    //! sensor is currently touched or not
    //*****************************************************************************
    bool bPresentTouch;

}tSensorGesture;


//*****************************************************************************
// Externals
//*****************************************************************************
extern gesture_t processSliderGesture(tSensor* pSensor);
extern gesture_t processWheelGesture(tSensor* pSensor);
extern gesture_t processButtonGroupGesture(tSensor* pSensor);

#endif /* GESTURES_H_ */
