/* --COPYRIGHT--,BSD
// BSD-3-Clause
//
// Copyright (c) 2018, Texas Instruments Incorporated
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
//
// * Redistributions of source code must retain the above copyright
// notice, this list of conditions and the following disclaimer.
//
// * Redistributions in binary form must reproduce the above copyright
// notice, this list of conditions and the following disclaimer in the
// documentation and/or other materials provided with the distribution.
//
// * Neither the name of Texas Instruments Incorporated nor the names of
// its contributors may be used to endorse or promote products derived
// from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
// PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
// OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
// WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
// OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
// EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/
//#############################################################################
//
//! \file
//!
//! \brief  Gesture algorithm for CapTIvate buttons
//
//  Group:          MSP
//  Target Devices: MSP430FR2633
//
//  (C) Copyright 2018, Texas Instruments, Inc.
//#############################################################################
// TI Release: 1.00.00.00
// Release Date: Dec 04, 2018
//#############################################################################

//*****************************************************************************
// Includes
//*****************************************************************************
#include "gestures.h"

//*****************************************************************************
// Definitions
//*****************************************************************************
#ifndef NONE
#define NONE         (0)
#endif

//*****************************************************************************
// Global Variables
//*****************************************************************************

//*****************************************************************************
// Function Prototypes (forward declarations)
//*****************************************************************************

//*****************************************************************************
// Local Variables - Gesture structures
//*****************************************************************************

//*****************************************************************************
// Function Implementations
//*****************************************************************************

#ifdef USE_BUTTONGROUP
// These parameters are for simple buttons.
// Modify and experiment with these values to achieve desired response.
tSensorGestureParams ButtonGroupGestureParams = {
     .ui16TouchSampleCount_Min = 3,            // default = 3
     .ui16TouchHoldSampleCount_Min = 8,        // default = 8, auto-repeat rate
};

tSensorGesture ButtonGroupGesture =
{
    .ui16SampleCount = 0,
    .params = (tSensorGestureParams*)&ButtonGroupGestureParams,
    .Gesture = eNoGesture,
    .TouchState = eStateBackGround,
    .bPresentTouch = false,
};

//*****************************************************************************
// Button Gestures
//*****************************************************************************
gesture_t processButtonGroupGesture(tSensor* pSensor)
{
    uint8_t ui8DominantButton;
     /*
      *  PART A: Process the slider sensor's touch/no-touch conditions.
      *  Need to know when sensor is touched, not touched and no change in touch.
      */

     if(FIRST_TOUCH)
     {
         // Set flag = sensor is touched.
         ButtonGroupGesture.bPresentTouch = true;

         // Reset sample counters.
         ButtonGroupGesture.ui16SampleCount = 0;

         // Increment sample counter on first touch
         ButtonGroupGesture.ui16SampleCount++;
     }
     else if(TOUCH)
     {
         // Increment sample counter during a touch
         ButtonGroupGesture.ui16SampleCount++;

     }else if(EXIT_TOUCH)
     {
         // Note: during EXIT_TOUCH or NO_TOUCH, sample counter is incremented
         // within the state machine (depending on the gesture).

         // Set flag = sensor is not touched
         ButtonGroupGesture.bPresentTouch = false;
     }

      /*
      *  PART B: Run through the state machine and determine gesture.
      */

     switch (ButtonGroupGesture.TouchState)
     {
     case eStateBackGround:

         if(ButtonGroupGesture.bPresentTouch == true)
         {
             // Check if minimum # of touch samples qualifies as a finger press
             // In this state, we will never see more samples than the tap sample minimum
             if(ButtonGroupGesture.ui16SampleCount == ButtonGroupGesture.params->ui16TouchSampleCount_Min)
             {
                 // Advance to the next state.
                 ButtonGroupGesture.TouchState = eStateFingerTap;
             }
         }

         // If sensor not presently touched then no gesture
         ButtonGroupGesture.Gesture = eNoGesture;

     break;

     case eStateFingerTap:

         if(ButtonGroupGesture.bPresentTouch == true)
         {
             if(ButtonGroupGesture.ui16SampleCount > ButtonGroupGesture.params->ui16TouchHoldSampleCount_Min)
             {
                  // Finger held too long to be a valid TAP.  Must be a hold.
                  ButtonGroupGesture.TouchState = eStateFingerHold;
             }
         }

         else
         {
             // Button released - Get the dominant button to know which button it is.
             ui8DominantButton = MAP_CAPT_getDominantButton(pSensor);

             switch(ui8DominantButton)
             {
             case 0:
                 ButtonGroupGesture.Gesture = eSwipeLeft;
                 break;
             case 1:
                 ButtonGroupGesture.Gesture = eSingleTap;
                 break;
             case 2:
                 ButtonGroupGesture.Gesture = eSwipeRight;
                 break;
             case 4:
                 ButtonGroupGesture.Gesture = eSlideLeft;
                 break;
             case 5:
                 ButtonGroupGesture.Gesture = eDoubleTap;
                 break;
             case 6:
                 ButtonGroupGesture.Gesture = eSlideRight;
                 break;
             }

             // Reset the state machine.
             ButtonGroupGesture.TouchState = eStateBackGround;
           }

         break;

     case eStateFingerHold:

        if(ButtonGroupGesture.bPresentTouch == true)
        {
            // Button continues to be touched - check if time to report gesture
            if(ButtonGroupGesture.ui16SampleCount > ButtonGroupGesture.params->ui16TouchHoldSampleCount_Min)
            {
                ButtonGroupGesture.ui16SampleCount = 0;

                ui8DominantButton = MAP_CAPT_getDominantButton(pSensor);

                switch(ui8DominantButton)
                {
                case 0:
                  ButtonGroupGesture.Gesture = eSwipeLeft;
                  ButtonGroupGesture.TouchState = eStateWaitFingerRelease;
                  break;
                case 1:
                  ButtonGroupGesture.Gesture = eSingleTap;
                  ButtonGroupGesture.TouchState = eStateWaitFingerRelease;
                  break;
                case 2:
                  ButtonGroupGesture.Gesture = eSwipeRight;
                  ButtonGroupGesture.TouchState = eStateWaitFingerRelease;
                  break;
                case 4:
                  ButtonGroupGesture.Gesture = eSlideLeft;
                  break;
                case 5:
                  ButtonGroupGesture.Gesture = eDoubleTap;
                  ButtonGroupGesture.TouchState = eStateWaitFingerRelease;
                  break;
                case 6:
                  ButtonGroupGesture.Gesture = eSlideRight;
                  break;
                default:
                  ButtonGroupGesture.Gesture = eNoGesture;
                }
            }
            else
                ButtonGroupGesture.Gesture = eNoGesture;

         }
         else
         {
             // Finger released so we are done here!
             ButtonGroupGesture.TouchState = eStateBackGround;
         }
         break;

     case eStateWaitFingerRelease:

         // Wait for finger release.  Then we can reset state machine
         if(ButtonGroupGesture.bPresentTouch == false)
             ButtonGroupGesture.TouchState = eStateBackGround;

         ButtonGroupGesture.Gesture = eNoGesture;
         break;

     default: break;

     }

     return(ButtonGroupGesture.Gesture);

}
#endif
