/* --COPYRIGHT--,BSD
// BSD-3-Clause
//
// Copyright (c) 2018, Texas Instruments Incorporated
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
//
// * Redistributions of source code must retain the above copyright
// notice, this list of conditions and the following disclaimer.
//
// * Redistributions in binary form must reproduce the above copyright
// notice, this list of conditions and the following disclaimer in the
// documentation and/or other materials provided with the distribution.
//
// * Neither the name of Texas Instruments Incorporated nor the names of
// its contributors may be used to endorse or promote products derived
// from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
// PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
// OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
// WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
// OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
// EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/
//#############################################################################
//
//! \file
//!
//! \brief  Gesture algorithm for CapTIvate sliders
//
//  Group:          MSP
//  Target Devices: MSP430FR2633
//
//  (C) Copyright 2018, Texas Instruments, Inc.
//#############################################################################
// TI Release: 1.00.00.00
// Release Date: Dec 04, 2018
//#############################################################################

//*****************************************************************************
// Includes
//*****************************************************************************
#include "gestures.h"

//*****************************************************************************
// Definitions
//*****************************************************************************
#ifndef NONE
#define NONE         (0)
#endif

//*****************************************************************************
// Global Variables
//*****************************************************************************

//*****************************************************************************
// Function Prototypes (forward declarations)
//*****************************************************************************

//*****************************************************************************
// Local Variables - Gesture structures
//*****************************************************************************
#ifdef USE_SLIDER
// These parameters are based on slider range = 1000 (default)
// Modify and experiment with these values to achieve desired response.
tSensorGestureParams SliderGestureParams = {
    .ui16TouchSampleCount_Min = 3,          // default = 3
    .ui16TouchSampleCount_Max = 16,         // default = 16
    .ui16SwipeSampleCount_Max = 10,         // default = 10
    .ui16DoubleTapDelayCount_Max = 10,      // default = 10
    .ui16FingerDistance_Min = 10,           // default = 10
    .ui16SlideStepSize_Min = 25,            // default = 25
    .ui16SwipeDistance_Min = 50 ,           // default = 50
};

tSensorGesture SliderGesture =
{
     .ui16SampleCount = 0,
     .ui16FingerFirstPos = 0,
     .ui16FingerPreviousPos = 0,
     .ui16FingerLastPos = 0,
     .ui16FingerDeltaPos = 0,
     .params = (tSensorGestureParams*)&SliderGestureParams,
     .Gesture = eNoGesture,
     .SwipeDirection = eNoGesture,
     .TouchState = eStateBackGround,
     .bPresentTouch = false,
};



//*****************************************************************************
// Function Implementations
//*****************************************************************************
static uint16_t absDelta(uint16_t ui16ValueA, uint16_t ui16ValueB)
{
    if(ui16ValueA > ui16ValueB)
    {
        return ui16ValueA - ui16ValueB;
    }
    else
    {
        return ui16ValueB - ui16ValueA;
    }
}


//*****************************************************************************
// Slider Gestures
//*****************************************************************************
gesture_t processSliderGesture(tSensor* pSensor)
{

    /*
     *  PART A: Process the slider sensor's touch/no-touch conditions.
     *  Need to know when sensor is touched, not touched and no change in touch.
     */

    if(FIRST_TOUCH)
    {
        // Set flag = sensor is touched.
        SliderGesture.bPresentTouch = true;

        // Reset sample counters.
        SliderGesture.ui16SampleCount = 0;

        // Record initial touch position on the slider.
        SliderGesture.ui16FingerFirstPos = ((tSliderSensorParams*)pSensor->pSensorParams)->SliderPosition.ui16Natural;
        SliderGesture.ui16FingerPreviousPos = 0;
        SliderGesture.ui16FingerLastPos = 0;

        // Increment sample counter on first touch
        SliderGesture.ui16SampleCount++;
    }
    else if(TOUCH)
    {
        /*
         * Here both current and previous samples == true so this is a continuous touch.
         * Since we won't know slider position if/when transition to a no touch, always
         * record current slider position and use as last position.
         */
        SliderGesture.ui16FingerLastPos = ((tSliderSensorParams*)pSensor->pSensorParams)->SliderPosition.ui16Natural;

        // Increment sample counter during a touch
        SliderGesture.ui16SampleCount++;

    }else if(EXIT_TOUCH)
    {
        // Note: during EXIT_TOUCH or NO_TOUCH, sample counter is incremented depending on the gesture within the state machine.

        // Set flag = sensor is not touched
        SliderGesture.bPresentTouch = false;

        // Calculate the delta from finger touch to finger release.
        SliderGesture.ui16FingerDeltaPos = absDelta(SliderGesture.ui16FingerLastPos, SliderGesture.ui16FingerFirstPos);

        // Set the direction
        if(SliderGesture.ui16FingerLastPos > SliderGesture.ui16FingerFirstPos)
            SliderGesture.SwipeDirection = eSwipeRight;
        else
            SliderGesture.SwipeDirection = eSwipeLeft;
    }

     /*
     *  PART B: Run through the state machine and determine gesture.
     */

    switch (SliderGesture.TouchState)
    {
    case eStateBackGround:

        if(SliderGesture.bPresentTouch == true)
        {
            // Check if minimum # of touch samples qualifies as a finger press
            // In this state, we will never see more samples than the tap sample minimum
            if(SliderGesture.ui16SampleCount == SliderGesture.params->ui16TouchSampleCount_Min)
            {
                // Advance to the next state.
                SliderGesture.TouchState = eStateFingerPress;
            }
        }

        // If sensor not presently touched then no gesture
        SliderGesture.Gesture = eNoGesture;

    break;


    case eStateFingerPress:

        //
        // Finger is touching here
        //
        if(SliderGesture.bPresentTouch == true)
        {
            // Has the finger moved?
            SliderGesture.ui16FingerDeltaPos = absDelta(SliderGesture.ui16FingerLastPos, SliderGesture.ui16FingerFirstPos);

            // If finger has moved in either direction more than the defined distance, advance to the slide state
            if(SliderGesture.ui16FingerDeltaPos > SliderGesture.params->ui16FingerDistance_Min)
            {
                SliderGesture.ui16FingerPreviousPos = SliderGesture.ui16FingerLastPos;

                // Reset the sample counter before advancing to the SLIDE state
                SliderGesture.ui16SampleCount = 0;

                SliderGesture.TouchState = eStateFingerSlide;
            }
        }

        else if(SliderGesture.bPresentTouch == false)
        {
            // Finger no longer touching at this point - is it a TAP?

            if(++SliderGesture.ui16SampleCount <= SliderGesture.params->ui16TouchSampleCount_Max)
            {
                // This is either a tap or could be a double-tap - advance to next state to figure out which one it is.
                SliderGesture.TouchState = eStateFingerTap;

                // Reset the sample counter before advancing to the TAP state
                // The sample counter will be used to measure the delay from end of TAP to beginning of DOUBLE TAP, if there is one.
                SliderGesture.ui16SampleCount = 0;
            }
            else
            {
                // Unfortunately this didn't qualify for any of the above so ignore and reset the state machine.
                SliderGesture.TouchState = eStateBackGround;
            }
        }
        break;

    case eStateFingerSlide:

        SliderGesture.ui16FingerDeltaPos = absDelta(SliderGesture.ui16FingerLastPos, SliderGesture.ui16FingerPreviousPos);

        if(SliderGesture.bPresentTouch == true)
        {
            if(SliderGesture.ui16SampleCount > SliderGesture.params->ui16SwipeSampleCount_Max)
            {
                // Finger has been touching too long to be a swipe, so must be slide
                // Has the finger moved the minimum step size?
                if(SliderGesture.ui16FingerDeltaPos < SliderGesture.params->ui16SlideStepSize_Min)
                {
                    // Finger hasn't move much so report it as no gesture
                    SliderGesture.Gesture = eNoGesture;
                }
                else
                {
                    // Finger moved so report it as a slider move gesture
                    if(SliderGesture.ui16FingerLastPos > SliderGesture.ui16FingerPreviousPos)
                        SliderGesture.Gesture = eSlideRight;
                    else
                        SliderGesture.Gesture = eSlideLeft;

                    SliderGesture.ui16FingerPreviousPos = SliderGesture.ui16FingerLastPos;
                }
            }
        }
        else if(SliderGesture.bPresentTouch == false)
        {
            // Did we cover the minimum swipe distance?
            if(SliderGesture.ui16FingerDeltaPos > SliderGesture.params->ui16SwipeDistance_Min)
            {
                if(++SliderGesture.ui16SampleCount < SliderGesture.params->ui16SwipeSampleCount_Max)
                {
                    // Swipe direction was determined at the beginning of this function call so simply post the gesture.
                    SliderGesture.Gesture = SliderGesture.SwipeDirection;
                }
            }

            // Set flag = sensor is not touched
            SliderGesture.bPresentTouch = false;

            // Done here so reset the state machine!
            SliderGesture.TouchState = eStateBackGround;
        }
        break;

    case eStateFingerTap:

        if(SliderGesture.bPresentTouch == true)
        {
            // Previous state finger not touching so this is transition into start of a second tap (double tap)
            SliderGesture.Gesture = eDoubleTap;

            SliderGesture.bPresentTouch = false;

            SliderGesture.TouchState = eStateBackGround;
        }

        else if(++SliderGesture.ui16SampleCount > SliderGesture.params->ui16DoubleTapDelayCount_Max)
        {
            // No second tap detected so report as a single tap event.
            SliderGesture.Gesture = eSingleTap;

            // Set flag = sensor is not touched
            SliderGesture.bPresentTouch = false;

            // Reset the state machine.
            SliderGesture.TouchState = eStateBackGround;
          }
        break;

    default:    break;

    }

    return(SliderGesture.Gesture);
}
#endif
