/* --COPYRIGHT--,BSD
// BSD-3-Clause
//
// Copyright (c) 2018, Texas Instruments Incorporated
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
//
// * Redistributions of source code must retain the above copyright
// notice, this list of conditions and the following disclaimer.
//
// * Redistributions in binary form must reproduce the above copyright
// notice, this list of conditions and the following disclaimer in the
// documentation and/or other materials provided with the distribution.
//
// * Neither the name of Texas Instruments Incorporated nor the names of
// its contributors may be used to endorse or promote products derived
// from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
// PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
// OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
// WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
// OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
// EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/
//#############################################################################
//
//! \file
//!
//! \brief  Gesture algorithm for CapTIvate wheels
//
//  Group:          MSP
//  Target Devices: MSP430FR2633
//
//  (C) Copyright 2018, Texas Instruments, Inc.
//#############################################################################
// TI Release: 1.00.00.00
// Release Date: Dec 04, 2018
//#############################################################################

//*****************************************************************************
// Includes
//*****************************************************************************
#include "gestures.h"

//*****************************************************************************
// Definitions
//*****************************************************************************
#ifndef NONE
#define NONE         (0)
#endif

//*****************************************************************************
// Global Variables
//*****************************************************************************

//*****************************************************************************
// Function Prototypes (forward declarations)
//*****************************************************************************

//*****************************************************************************
// Local Variables - Gesture structures
//*****************************************************************************
#ifdef USE_WHEEL
// These parameters are based on slider range = 100 (default)
// Modify and experiment with these values to achieve desired response.
tSensorGestureParams WheelGestureParams =
{
    .ui16TouchSampleCount_Min = 3,          // default = 3
    .ui16TouchSampleCount_Max = 16,         // default = 16
    .ui16SwipeSampleCount_Max = 10,         // default = 10
    .ui16FingerDistance_Min = 3,            // default = 3
    .ui16SwipeDistance_Min = 10,            // default = 10
    .ui16SlideStepSize_Min = 5,             // default = 5
};

tSensorGesture WheelGesture =
{
    .ui16SampleCount = 0,
    .ui16FingerFirstPos = 0,
    .ui16FingerPreviousPos = 0,
    .ui16FingerLastPos = 0,
    .ui16FingerDeltaPos = 0,
    .params = (tSensorGestureParams*)&WheelGestureParams,
    .Gesture = eNoGesture,
    .SwipeDirection = eNoGesture,
    .TouchState = eStateBackGround,
    .bPresentTouch = false,
};


//*****************************************************************************
// Function Implementations
//*****************************************************************************
static uint16_t absDelta(uint16_t ui16ValueA, uint16_t ui16ValueB)
{
    if(ui16ValueA > ui16ValueB)
    {
        return ui16ValueA - ui16ValueB;
    }
    else
    {
        return ui16ValueB - ui16ValueA;
    }
}


//*****************************************************************************
// Wheel Gestures
//*****************************************************************************

gesture_t processWheelGesture(tSensor* pSensor)
{
    uint8_t ui8FingerPosition;

    /*
     *  PART A: Process the slider sensor's touch/no-touch conditions.
     *  Need to know when sensor is touched, not touched and no change in touch.
     */

    if(FIRST_TOUCH)
    {
        // Set flag = sensor is touched.
        WheelGesture.bPresentTouch = true;

        // Reset sample counters.
        WheelGesture.ui16SampleCount = 0;

        // Record initial touch position on the slider.
        WheelGesture.ui16FingerFirstPos = ((tSliderSensorParams*)pSensor->pSensorParams)->SliderPosition.ui16Natural;
        WheelGesture.ui16FingerPreviousPos = 0;
        WheelGesture.ui16FingerLastPos = 0;

        // Increment sample counter on first touch
        WheelGesture.ui16SampleCount++;
    }
    else if(TOUCH)
    {
        /*
         * Here both current and previous samples == true so this is a continuous touch.
         * Since we won't know slider position if/when transition to a no touch, always
         * record current slider position and use as last position.
         */
        WheelGesture.ui16FingerLastPos = ((tSliderSensorParams*)pSensor->pSensorParams)->SliderPosition.ui16Natural;

        // Increment sample counter during a touch
        WheelGesture.ui16SampleCount++;

    }else if(EXIT_TOUCH)
    {
        // Note: during EXIT_TOUCH or NO_TOUCH, sample counter is incremented depending on the gesture within the state machine.

        // Set flag = sensor is not touched
        WheelGesture.bPresentTouch = false;

        // Calculate the delta from finger touch to finger release.
        WheelGesture.ui16FingerDeltaPos = absDelta(WheelGesture.ui16FingerLastPos, WheelGesture.ui16FingerFirstPos);

        // Set the direction
        if(WheelGesture.ui16FingerLastPos > WheelGesture.ui16FingerFirstPos)
            WheelGesture.SwipeDirection = eSwipeRight;
        else
            WheelGesture.SwipeDirection = eSwipeLeft;
    }

     /*
     *  PART B: Run through the state machine and determine gesture.
     */

    switch (WheelGesture.TouchState)
    {
    case eStateBackGround:

        if(WheelGesture.bPresentTouch == true)
        {
            // Check if minimum # of touch samples qualifies as a finger press
            // In this state, we will never see more samples than the tap sample minimum
            if(WheelGesture.ui16SampleCount == WheelGesture.params->ui16TouchSampleCount_Min)
            {
                // Advance to the next state.
                WheelGesture.TouchState = eStateFingerPress;
            }
        }

        // If sensor not presently touched then no gesture
        WheelGesture.Gesture = eNoGesture;

    break;


    case eStateFingerPress:

        if(WheelGesture.bPresentTouch == true)
        {
            // Has the finger moved?
            WheelGesture.ui16FingerDeltaPos = absDelta(WheelGesture.ui16FingerLastPos, WheelGesture.ui16FingerFirstPos);

            // If finger has moved in either direction more than the defined distance, advance to the slide state
            if(WheelGesture.ui16FingerDeltaPos > WheelGesture.params->ui16FingerDistance_Min)
            {
                WheelGesture.ui16FingerPreviousPos = WheelGesture.ui16FingerLastPos;

                // Reset the sample counter before advancing to the SLIDE state
                WheelGesture.ui16SampleCount = 0;

                WheelGesture.TouchState = eStateFingerSlide;
            }
        } // end of finger touching

        else if(WheelGesture.bPresentTouch == false)
        {
            // Finger no longer touching at this point - is it a TAP?
            if(++WheelGesture.ui16SampleCount > WheelGesture.params->ui16TouchSampleCount_Max)
            {
                // Finger was held too long
                WheelGesture.TouchState = eStateBackGround;

                // Set flag = sensor is not touched
                WheelGesture.bPresentTouch = false;
            }
            else
            {
                ui8FingerPosition = WheelGesture.ui16FingerLastPos;

               // What position on the wheel we are?
               if(((ui8FingerPosition >= 95) && (ui8FingerPosition <= 99)) || (ui8FingerPosition <= 5))
                   WheelGesture.Gesture = eDoubleTap; // mute

               else if((ui8FingerPosition >= 20) && (ui8FingerPosition <= 30))
                   WheelGesture.Gesture = eSwipeRight; // next track

               else if((ui8FingerPosition >= 45) && (ui8FingerPosition <= 55))
                   WheelGesture.Gesture = eSingleTap; // play/pause

               else if((ui8FingerPosition >= 70) && (ui8FingerPosition <= 80))
                   WheelGesture.Gesture = eSwipeLeft; // previous track

               // Set flag = sensor is not touched
               WheelGesture.bPresentTouch = false;

               // Reset the state machine.
               WheelGesture.TouchState = eStateBackGround;
            }
        }
        break;

    case eStateFingerSlide:

        WheelGesture.ui16FingerDeltaPos = absDelta(WheelGesture.ui16FingerLastPos, WheelGesture.ui16FingerPreviousPos);

        if(WheelGesture.bPresentTouch == true)
        {
            if(WheelGesture.ui16SampleCount > WheelGesture.params->ui16SwipeSampleCount_Max)
            {
                // Finger has been touching too long to be a swipe, so must be slide
                // Has the finger moved the minimum step size?
                if(WheelGesture.ui16FingerDeltaPos < WheelGesture.params->ui16SlideStepSize_Min)
                {
                    // Finger hasn't move much so report it as no gesture
                    WheelGesture.Gesture = eNoGesture;
                }
                else
                {
                    // Finger moved so report it as a slider move gesture
                    if(WheelGesture.ui16FingerLastPos > WheelGesture.ui16FingerPreviousPos)
                        WheelGesture.Gesture = eSlideRight;
                    else
                        WheelGesture.Gesture = eSlideLeft;

                    WheelGesture.ui16FingerPreviousPos = WheelGesture.ui16FingerLastPos;
                }
            }
        }
        else if(WheelGesture.bPresentTouch == false)
        {
            // Finger no longer touching.
            // Did we cover the minimum swipe distance?
            if(WheelGesture.ui16FingerDeltaPos > WheelGesture.params->ui16SwipeDistance_Min)
            {
                // And did we do it in less time than the max?
                if(++WheelGesture.ui16SampleCount < WheelGesture.params->ui16SwipeSampleCount_Max)
                {
                    // Swipe direction was determined at the beginning of this function call so simply post the gesture.
                    WheelGesture.Gesture = WheelGesture.SwipeDirection;
                }
            }

            // Set flag = sensor is not touched
            WheelGesture.bPresentTouch = false;

            // Done here so reset the state machine!
            WheelGesture.TouchState = eStateBackGround;
        }
        break;

    default: break;;

    }

    return(WheelGesture.Gesture);
}

#endif
