/* --COPYRIGHT--,BSD
// BSD-3-Clause
//
// Copyright (c) 2018, Texas Instruments Incorporated
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
//
// * Redistributions of source code must retain the above copyright
// notice, this list of conditions and the following disclaimer.
//
// * Redistributions in binary form must reproduce the above copyright
// notice, this list of conditions and the following disclaimer in the
// documentation and/or other materials provided with the distribution.
//
// * Neither the name of Texas Instruments Incorporated nor the names of
// its contributors may be used to endorse or promote products derived
// from this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
// PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
// OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
// WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
// OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
// EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/
//#############################################################################
//
//! \file   keyboard.c
//!
//! \brief  Accepts Page7 HID Keyboard commands and sends via USB to host.
//
//  Group:          MSP
//  Target Devices: MSP430F5529
//
//  (C) Copyright 2018, Texas Instruments, Inc.
//#############################################################################
// TI Release: 1.0.00.00
// Release Date: Dec 4, 2018
//#############################################################################

//*****************************************************************************
// Includes
//*****************************************************************************
#include <stdint.h>
#include <stdlib.h>
#include "USB_config/descriptors.h"
#include "USB_API/USB_Common/device.h"
#include "USB_API/USB_Common/usb.h"
#include "USB_API/USB_HID_API/UsbHid.h"
#include <driverlib.h>
#include "board.h"
#include "keyboard.h"

//*****************************************************************************
// Global Variables
//*****************************************************************************
volatile bool g_bKeySendComplete = true;

//*****************************************************************************
// Local Variables
//*****************************************************************************
static  KeyUnion keyUnion;
bool Keyboard_IRQ = false;


//*****************************************************************************
// Function Implementations
//*****************************************************************************
void Keyboard_start(void)
{
	uint8_t i;

	// Initialize the key report
	for (i=0; i<8; i++) {
		keyUnion.keyArray[i] = 0;
	}

	// Configure I2C
	USCI_B_I2C_initMasterParam USCIBConfig =
    {
        .selectClockSource = USCI_B_I2C_CLOCKSOURCE_SMCLK,
        .i2cClk = 24000000,
        .dataRate = USCI_B_I2C_SET_DATA_RATE_400KBPS,
    };

    USCI_B_I2C_initMaster(USCI_B1_BASE, &USCIBConfig);
    USCI_B_I2C_disable(USCI_B1_BASE);
    USCI_B_I2C_enable(USCI_B1_BASE);
    USCI_B_I2C_setSlaveAddress(USCI_B1_BASE, 0x0A);

    GPIO_selectInterruptEdge(BOARD_KEYBOARD_IRQ_PORT, BOARD_KEYBOARD_IRQ_PIN, GPIO_HIGH_TO_LOW_TRANSITION);
    GPIO_clearInterrupt(BOARD_KEYBOARD_IRQ_PORT, BOARD_KEYBOARD_IRQ_PIN);
    GPIO_enableInterrupt(BOARD_KEYBOARD_IRQ_PORT, BOARD_KEYBOARD_IRQ_PIN);
}


void Keyboard_sendReport()
{
    USBHID_sendReport( keyUnion.keyArray, HID0_INTFNUM);
}


// function Keyboard_press() adds the specified key (printing, non-printing, or modifier)
// to the persistent key report and sends the report.  Because of the way 
// USB HID works, the host acts like the key remains pressed until we 
// call release(), releaseAll(), or otherwise clear the report and resend.
bool Keyboard_press(uint16_t key)
{
    uint8_t i;
    uint8_t k;

    k = (uint8_t)(key & 0x00FF);

    // Check for modifier key (SHIFT, CTRL, ALT, META...)
    if((key & 0xE000) ==  0xE000)
        keyUnion.keyReport.modifiers |= (1 << (uint8_t)((key & 0x0700) >> 8));

    // Add k to the key report only if it's not already present
    // and if there is an empty slot.
    if (keyUnion.keyReport.keys[0] != k && keyUnion.keyReport.keys[1] != k &&
        keyUnion.keyReport.keys[2] != k && keyUnion.keyReport.keys[3] != k &&
        keyUnion.keyReport.keys[4] != k && keyUnion.keyReport.keys[5] != k) {

        for (i=0; i<6; i++) {
            if (keyUnion.keyReport.keys[i] == 0x00) {
                keyUnion.keyReport.keys[i] = k;
                break;
            }
        }
        if (i == 6) {
            return 0;
        }
    }
    Keyboard_sendReport();
    return 1;
}

// Function keyboard_release() takes the specified key out of the persistent key report and
// sends the report.  This tells the OS the key is no longer pressed and that
// it shouldn't be repeated any more.
bool Keyboard_release(uint16_t key)
{
    uint8_t i;
    uint8_t k;

    k = (uint8_t)(key & 0x00FF);
    keyUnion.keyReport.modifiers &= ~((key & 0xFF00)>>8);

    // Check for modifier key (SHIFT, CTRL, ALT, META...)
    if((key & 0xE000) ==  0xE000)
        keyUnion.keyReport.modifiers &= ~(1 << (uint8_t)((key & 0x0700) >> 8));


    // Test the key report to see if k is present.  Clear it if it exists.
    // Check all positions in case the key is present more than once
    // (which it shouldn't be)
    for (i=0; i<6; i++) {
        if (0 != k && keyUnion.keyReport.keys[i] == k) {
            keyUnion.keyReport.keys[i] = 0x00;
        }
    }

    Keyboard_sendReport();
    return 1;
}

bool Keyboard_getIRQStatus(void)
{
    if(Keyboard_IRQ == true)
    {
        Keyboard_IRQ = false;
        return true;
    }

    return false;
}

void Keyboard_read(uint16_t *currentStatus, uint16_t *previousStatus)
{
    uint8_t data = 0;

    USCI_B_I2C_setMode(USCI_B1_BASE, USCI_B_I2C_TRANSMIT_MODE);
    USCI_B_I2C_clearInterrupt(USCI_B1_BASE, USCI_B_I2C_RECEIVE_INTERRUPT);
    USCI_B_I2C_clearInterrupt(USCI_B1_BASE, USCI_B_I2C_TRANSMIT_INTERRUPT);
    while (USCI_B_I2C_isBusBusy(USCI_B1_BASE));

    USCI_B_I2C_masterReceiveMultiByteStart(USCI_B1_BASE);

    while(!(USCI_B_I2C_getInterruptStatus(USCI_B1_BASE, USCI_B_I2C_RECEIVE_INTERRUPT)));
    data = USCI_B_I2C_masterReceiveMultiByteNext(USCI_B1_BASE);
    *currentStatus = data & 0x00FF;

    while(!(USCI_B_I2C_getInterruptStatus(USCI_B1_BASE, USCI_B_I2C_RECEIVE_INTERRUPT)));
    data = USCI_B_I2C_masterReceiveMultiByteNext(USCI_B1_BASE);
    *currentStatus |= (data << 8) & 0xFF00;

    while(!(USCI_B_I2C_getInterruptStatus(USCI_B1_BASE, USCI_B_I2C_RECEIVE_INTERRUPT)));
    data = USCI_B_I2C_masterReceiveMultiByteFinish(USCI_B1_BASE);
    *previousStatus = data & 0x00FF;
    data = USCI_B_I2C_masterReceiveMultiByteNext(USCI_B1_BASE);
    *previousStatus |= (data << 8) & 0xFF00;
}

void Demo_updateHIDReport(uint16_t ui16CurrentKey, uint16_t ui16PreviousKey)
{

    // Test state change to signal the report needs updating.
    if (ui16CurrentKey != ui16PreviousKey)
    {

        g_bKeySendComplete = false;

        //Check for a key
        if(LOBYTE(ui16CurrentKey))
        {
            Keyboard_press(ui16CurrentKey);
        }
        else
        {
            // Use previous key
            Keyboard_release(ui16PreviousKey);
        }

        while(!g_bKeySendComplete);

    }

}

#pragma vector=PORT2_VECTOR
__interrupt void Port_2_ISR(void)
{
    if(GPIO_getInterruptStatus(BOARD_KEYBOARD_IRQ_PORT, BOARD_KEYBOARD_IRQ_PIN) != 0)
    {
        GPIO_clearInterrupt(BOARD_KEYBOARD_IRQ_PORT, BOARD_KEYBOARD_IRQ_PIN);
        Keyboard_IRQ = true;
    }

    __bic_SR_register_on_exit(LPM4_bits);
}


